from .version import Version
from .project import Project

class Local(Project):
    '''
    Object representing a the current project installed by the user.
    '''
    def __init__(self, author: str, projectName: str, version: str, separator: str):
        version = Version(version, separator)
        super().__init__(author, projectName, version)



class Version:
    '''
    Object representing a version number used to make handling Project versions easier
    '''
    __slots__ = ['__version_list', '__version_str']


    def __init__(self, version='0.0.0', separator='.'):
        self.__version_str = version
        self.__version_list = list(version.split(separator))


    def __str__(self) -> str:
        return self.__version_str


    def __repr__(self) -> str:
        return f'Version[{self.__version_str}, {self.__version_list}]'


    def __lt__(self, other) -> bool:
        '''
        Compare each version category to tell if local version is less than, equal to, or greater than the latest GitHub release
        '''
        for i in range(len(self.__version_list)):
            version_number = int(self.__version_list[i])
            latest_version_number = int(other.__version_list[i])
            if version_number < latest_version_number:
                return True
            elif version_number > latest_version_number:
                return False
        return False


    def __eq__(self, other) -> bool:
        if type(other) != Version:
            return False

        return repr(self) == repr(other)



class Event:
    __slots__ = ['__funcs']


    def __init__(self):
        self.__funcs = []


    def __call__(self) -> list:
        for func in self.__funcs:
            func()


    def __iadd__(self, func):
        self.__funcs.append(func)
        return self



class Project:
    '''
    Base class for a project.
    '''
    __slots__ = ['__author', '__name', '__version']

    def __init__(self, author, name, version):
        self.__author = author
        self.__name = name
        self.__version = version


    def __repr__(self):
        return f'Project[{self.__author}, {self.__name}, {self.__version}]'


    def set_version(self, version):
        self.__version = version


    def verison(self):
        return self.__version



import requests

from .project import Project
from .version import Version

class Remote (Project):
    '''
    Object representing a the latest project version on github.
    '''
    __slots__ = ['__url', '__separator', '__data']


    def __init__(self, author: str, projectName: str, separator: str):
        self.__url = f'https://api.github.com/repos/{author}/{projectName}/releases/latest'
        self.__separator = separator
        self.__data = requests.get(self.__url).json()

        version = Version(self.__data['tag_name'], self.__separator)
        super().__init__(author, projectName, version)


    def refresh(self):
        self.__data = requests.get(self.__url).json()
        version = Version(self.__data['tag_name'], self.__separator)
        super().set_version(version)


    def get_data(self):
        return self.__data



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    # the name must match the folder name 'verysimplemodule'
    name="versionmanagerpy",
    version='1.0.1',
    author="Kamron Cole",
    author_email="kjc8084@rit.edu",
    description='A GitHub Project Version Manager.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/NeonixRIT/versionmanager.py",
    project_urls={
        "Bug Tracker": "https://github.com/NeonixRIT/versionmanager.py/issues",
    },
    packages=find_packages(),
    install_requires=[],
    keywords=['python', 'github', 'version', 'manager', 'version manager'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.6",
)



'''
A GitHub Project Version Manager that polls latest version data from GitHub repo release tag

author: Kamron Cole (kjc8084@rit.edu)
'''
from enum import Enum
from .event import Event
from .local import Local
from .remote import Remote

class Status(Enum):
    OUTDATED = -1,
    CURRENT = 0,
    DEV = 1


class VersionManager:
    '''
    The main class to compare a local project to the latest Github release and perform actions based on the result
    '''
    __slots__ = ['__local', '__remote', '__author', '__projectName', '__separator', 'outdated_event', 'current_event', 'dev_event']


    def __init__(self, author: str, projectName: str, version: str, separator='.'):
        '''
        Instantiate the VersionManager for a project
        author: the owner of the project's repo
        projectName: the name of the project's repo
        version: the semantic version of the project as a string
        separator: the separator used to separate the version categories
        '''
        self.__author = author
        self.__projectName = projectName
        self.__separator = separator

        self.__local = Local(author, projectName, version, separator)
        self.__remote = None

        self.outdated_event = Event()
        self.current_event = Event()
        self.dev_event = Event()


    def check_status(self) -> Status:
        '''
        Compare local project version to the latest GitHub release version. Trigger events based on comparison result
        returns: Result of version comparison in form of values from the Status Enum
        '''
        if self.__remote is None:
            self.__remote = Remote(self.__author, self.__projectName, self.__separator)
        else:
            self.__remote.refresh()

        if self.__local.verison() == self.__remote.verison():
            self.current_event()
            return Status.CURRENT
        elif self.__local.verison() < self.__remote.verison():
            self.outdated_event()
            return Status.OUTDATED
        else:
            self.dev_event()
            return Status.DEV


    def get_data(self) -> dict:
        '''
        Get various other information about the latest GitHub release from the parsed JSON
        returns: A dictionary representation of the JSON
        '''
        if self.__remote is None:
            self.__remote = Remote(self.__author, self.__projectName, self.__separator)
        return self.__remote.get_data()
